/*

  Support for scripting of plugins using the npruntime interface

  Copyright (c) 2006, 2010 Maksim Orlovich <maksim@kde.org>
  Copyright (c) 2009 by Marco Martin <notmart@gmail.com>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*/

#include "dbustypes.h"
#include <QDBusMetaType>

const QDBusArgument& operator<<(QDBusArgument& arg, const NSLiveConnectResult& res)
{
    arg.beginStructure();
    arg << res.success;
    arg << res.type;
    arg << res.objid;
    arg << res.value;
    arg.endStructure();
    return arg;
}

const QDBusArgument& operator>>(const QDBusArgument& arg, NSLiveConnectResult& res)
{
    arg.beginStructure();
    arg >> res.success;
    arg >> res.type;
    arg >> res.objid;
    arg >> res.value;
    arg.endStructure();
    return arg;
}

namespace kdeNsPluginViewer  {

void initDBusTypes()
{
    qDBusRegisterMetaType<NSLiveConnectResult>();
}

}

// kate: indent-width 4; replace-tabs on; tab-width 4; space-indent on;
