<!DOCTYPE kpartgui>
<kpartgui name="khtmlkttsdplugin" library="khtmlkttsdplugin" version="3" >
	<MenuBar>
		<Menu name="tools">
		<text>&amp;Tools</text>
			<Action name="tools_kttsd"/>
		</Menu>
	</MenuBar>
	<ToolBar name="speechToolbar" hidden="true">
		<text>Speech Toolbar</text>
		<Action name="tools_kttsd"/>
	</ToolBar>
</kpartgui>

