<!DOCTYPE kpartgui>
<gui name="domtreeviewer" library="domtreeviewerplugin" version="1">
<MenuBar>
 <Menu name="tools"><text>&amp;Tools</text>
  <Action name="viewdomtree"/>
 </Menu>
</MenuBar>
<ToolBar name="extraToolBar"><text>Extra Toolbar</text>
  <Action name="viewdomtree"/>
</ToolBar>
</gui>
