/*  This file is part of the KDE project
    Copyright 2009  Harald Hvaal <haraldhv@stud.ntnu.no>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) version 3.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

#include "konq_dndpopupmenuplugin.h"

KonqDndPopupMenuPlugin::KonqDndPopupMenuPlugin(QObject* parent)
    : QObject(parent)
{
}

KonqDndPopupMenuPlugin::~KonqDndPopupMenuPlugin()
{
}
